@@ -2,6 +2,8 @@ import logging
 import traceback
 
 from deepdiff import DeepDiff, Delta
+import zmq
+from uuid import uuid4
 
 from .constants import ADDED, COMMITED, FETCHED, MODIFIED, RP_COMMON, UP
 from .exception import (NetworkFrameError,
@@ -9,7 +11,8 @@ from .exception import (NetworkFrameError,
                         NonAuthorizedOperationError)
 from .objects import (Node, ReplicationObject, Auth, Delete, Disconnect,
                       Kick, Right, ServerSnapshot, Snapshot, Commit,
-                      UpdateClientsState, UpdateUserMetadata)
+                      UpdateClientsState, UpdateUserMetadata,
+                      RequestServerInfo)
 from .repository import Remote, Repository
 
 
@@ -195,6 +198,7 @@ def remote_add(repository: Repository,
                name: str,
                address: str,
                port: int,
+               server_password: str = None,
                admin_password: str = None,
                realtime: bool = False):
     """ Add a new distant server remote to the repository
@@ -216,6 +220,7 @@ def remote_add(repository: Repository,
         repository.remotes[name] = Remote(name=name,
                                           address=address,
                                           port=port,
+                                          server_password=server_password,
                                           admin_password=admin_password,
                                           realtime=realtime)
         if not rt_remotes:
@@ -350,6 +355,7 @@ def unlock(repository: Repository,
 
     repository.push(repository.remote.command, right_command)
 
+
 def kick(repository: Repository, username: str):
     """Kick a user from the active realtime remote.
 
@@ -435,4 +441,42 @@ def purge_orphan_nodes(repository: Repository):
         except NonAuthorizedOperationError:
             logging.warning(f"Skipping node {node} removal")
         else:
-            rm(repository, node, remove_dependencies=False)
\ No newline at end of file
+            rm(repository, node, remove_dependencies=False)
+
+def request_session_info(address: str, timeout: int = 100):
+    """Retrieve the target session informations in the following format:
+        {
+          "private": str, is the session protected by a password
+          "version": str, server version
+        }
+
+        :param address: server address (ip:port)
+        :type address: str
+        :return: dict
+    """
+    session_info = None
+
+    context = zmq.Context()
+    command = context.socket(zmq.DEALER)
+    command.setsockopt(zmq.IDENTITY, uuid4().bytes)
+    command.connect(f"tcp://{address}")
+    command.linger = 0
+    poller = zmq.Poller()
+    poller.register(command, zmq.POLLIN)
+
+    info_request = RequestServerInfo(
+        owner='guest',
+        data='RQ_INFO'
+    )
+
+    command.send_multipart(info_request.as_raw_chunks())
+
+    socks = dict(poller.poll(timeout))
+    if command in socks:
+        frame = command.recv_multipart(0)
+        session_info_frame = ReplicationObject.from_raw_chunks(frame)
+        session_info = session_info_frame.data
+    command.close()
+    context.destroy()
+
+    return session_info
\ No newline at end of file
