@@ -57,12 +57,15 @@ from replication.constants import (CLIENT_PING_FREQUENCY, CONNECTION_TIMEOUT,
                                    RP_COMMON, STATE_ACTIVE, STATE_AUTH,
                                    STATE_INITIAL, STATE_LOBBY, STATE_SRV_SYNC,
                                    STATE_SYNCING, STATE_WAITING, UP)
-from replication.objects import Node, ReplicationObject, Auth, Delete, Disconnect, Kick,Right, ServerSnapshot, Snapshot,UpdateClientsState, UpdateUserMetadata
+from replication.objects import (Node, ReplicationObject, Auth,
+                                 Delete, Disconnect, Kick,Right,
+                                 ServerSnapshot, Snapshot,UpdateClientsState,
+                                 UpdateUserMetadata, RequestServerInfo)
+
 from replication.exception import DataError, NetworkFrameError, StateError
 from replication.repository import Repository
 from replication.utils import current_milli_time
 
-
 class ServerNetService(threading.Thread):
     def __init__(self):
 
@@ -83,10 +86,12 @@ class ServerNetService(threading.Thread):
     def listen(
             self,
             port=5560,
-            password='admin',
+            server_password=None,
+            admin_password='admin',
             attached=False,
             timeout=CONNECTION_TIMEOUT):
-        self._password = password
+        self._server_password = server_password
+        self._admin_password = admin_password
         self._port = port
         self._attached = attached
 
@@ -122,31 +127,31 @@ class ServerNetService(threading.Thread):
         self.start()
 
     def disconnect_client(self, client, reason):
-        if client in self.clients:
-            leaving_client = self.clients[client]
-            cleanup_commands = []
-            for key, node in self._repository.object_store.items():
-                if node.owner == leaving_client['id']:
-                    logging.debug(f"Changing node {node.uuid} rights to COMMON")
-                    cleanup_commands.append(
-                        Right(
-                            owner='server',
-                            data={
-                                'uuid': node.uuid,
-                                'owner': RP_COMMON
-                            }
-                        ))
-            for rr_cmd in cleanup_commands:
+        leaving_client = self.clients.get(client)
+        if leaving_client:
+            client_id = leaving_client.get('id')
+            locked_nodes = [n.uuid for n in self._repository.graph.values() if n.owner == client_id]
+
+            if locked_nodes:
+                reset_right = Right(
+                                owner='server',
+                                data={
+                                    'uuid': locked_nodes,
+                                    'owner': RP_COMMON
+                                }
+                            )
+
+                logging.debug(f"Changing node {locked_nodes} rights to COMMON")
+                
                 for cli in self.clients.keys():
                     if cli != client and not self._command._closed:
                         self._command.send(cli, zmq.SNDMORE)
-                        self._repository.push(self._command, rr_cmd)
-                        rr_cmd.execute(self._repository.object_store)
+                        self._repository.push(self._command, reset_right)
+                        reset_right.execute(self._repository.object_store)
 
             del self.clients[client]
-            logging.info(f"{leaving_client['id']} disconnected from the server [{reason}]")
-            # if len(self.clients) == 0 and self._attached:
-            #     self.stop()
+            logging.info(f"{client_id} disconnected from the server [{reason}]")
+
 
     def _update_clients_states(self):
         user_dict = {}
@@ -163,7 +168,7 @@ class ServerNetService(threading.Thread):
             self._command.send(client_uid, zmq.SNDMORE)
             self._repository.push(self._command, clients_states)
 
-    def _login_client(self, auth_type, uid, id, password, version):
+    def _login_client(self, auth_type, uid, id, server_password, admin_password, version):
         """ Register a client on the server
 
         return:
@@ -177,8 +182,11 @@ class ServerNetService(threading.Thread):
                 logging.debug("client logged in")
                 return 'FAILED: client already logged in'
 
-        if auth_type == 'ADMIN' and password != self._password:
-            return 'FAILED: wrong password'
+        if auth_type == 'ADMIN' and admin_password != self._admin_password:
+            return 'FAILED: wrong admin password'
+
+        if str(server_password) != self._server_password:
+            return 'FAILED: wrong server password'
 
         if version != __version__:
             return f'FAILED: wrong client version ({version} != {__version__})'
@@ -229,7 +237,7 @@ class ServerNetService(threading.Thread):
             data={
                 'STATE': 'INIT',
                 'CATALOG': catalog})
-        logging.info(f"Pushing nodes to {self.clients[client]['id']}")
+        logging.info(f"{self.clients[client]['id']} - Snapshot : Pushing nodes catalog")
         self._command.send(client, zmq.SNDMORE)
         self._repository.push(self._command, snapshot_state)
 
@@ -237,26 +245,36 @@ class ServerNetService(threading.Thread):
         """ Handle client snapshot commands """
 
         snapshot_state = command.data.get('STATE')
+        snapshot_client = self.clients.get(command.sender)
 
-        if snapshot_state == 'REQUEST_INIT':
-            self.send_client_snapshot_init(command.sender)
-        elif snapshot_state == 'GET':
-            node = self._repository.graph.get(command.data['ID'])
-            snapshot_cmd = Snapshot(
-                    owner='server',
-                    data={
-                        'STATE': 'SET',
-                        'DATA': node.as_raw_chunks()
-                    }
-            )
-            self._command.send(command.sender, zmq.SNDMORE)
-            self._repository.push(self._command, snapshot_cmd)
-            logging.debug(f"pushing node {node.uuid} to {self.clients[command.sender]['id']}")
-        elif snapshot_state == 'DONE':
-            # Set client ready
-            logging.info(f"{self.clients[command.sender]['id']} up to date.")
-            self.clients[command.sender]['status'] = STATE_ACTIVE
-            self._update_clients_states()
+        if snapshot_client:
+            if snapshot_state == 'REQUEST_INIT':
+                self.send_client_snapshot_init(command.sender)
+            elif snapshot_state == 'GET':
+                node = self._repository.graph.get(command.data['ID'])
+
+                if node:
+                    logging.debug(f"{snapshot_client['id']} - Snapshot : pushing node {node.uuid}")
+                else:
+                    logging.debug(f"{snapshot_client['id']} - Snapshot : node {command.data['ID']} no longer exist, skipping it.")
+
+                node_data = node.as_raw_chunks() if node else b'removed'
+                snapshot_cmd = Snapshot(
+                        owner='server',
+                        data={
+                            'STATE': 'SET',
+                            'DATA': node_data
+                        }
+                )
+                self._command.send(command.sender, zmq.SNDMORE)
+                self._repository.push(self._command, snapshot_cmd)
+            elif snapshot_state == 'DONE':
+                # Set client ready
+                logging.info(f"{snapshot_client['id']} - Snapshot : done.")
+                self.clients[command.sender]['status'] = STATE_ACTIVE
+                self._update_clients_states()
+        else:
+            logging.error(f"Snapshot cancelled {command.sender} (client disconnected)")
 
     def handle_server_repository_init(self, command):
         cli_snapshot_state = command.data.get('STATE')
@@ -336,13 +354,15 @@ class ServerNetService(threading.Thread):
                         auth_type = command.data.get('AUTH_TYPE')
                         auth_origin = command.sender
                         auth_id = command.data.get('AUTH_ID')
-                        auth_pass = command.data.get('PWD', None)
+                        auth_s_pass = command.data.get('S_PWD', None)
+                        auth_a_pass = command.data.get('A_PWD', None)
                         client_version = command.data.get('VERSION')
 
                         auth_status = self._login_client(auth_type,
                                                         auth_origin,
                                                         auth_id,
-                                                        auth_pass,
+                                                        auth_s_pass,
+                                                        auth_a_pass,
                                                         client_version)
 
                         auth_response = Auth(
@@ -382,6 +402,16 @@ class ServerNetService(threading.Thread):
                     if isinstance(command, Kick):
                         self.kick(command.data['user'])
 
+                    if isinstance(command, RequestServerInfo):
+                        server_info = RequestServerInfo(
+                            owner='server',
+                            data={
+                                'private': True if self._server_password else False,
+                                'version': __version__
+                            })
+                        self._command.send(command.sender, zmq.SNDMORE)
+                        self._command.send_multipart(server_info.as_raw_chunks())
+
                     # OTHERS
                     if type(command) in [Delete, Right]:
                         try:
@@ -605,7 +635,8 @@ class ServerShell(cmd.Cmd):
             self,
             port=5555,
             timeout=CONNECTION_TIMEOUT,
-            password='None',
+            server_password=None,
+            admin_password=None,
             attached=False):
 
         cmd.Cmd.__init__(self)
@@ -613,7 +644,8 @@ class ServerShell(cmd.Cmd):
 
         self._net.listen(
             port=port,
-            password=password,
+            server_password=server_password,
+            admin_password=admin_password,
             attached=attached,
             timeout=timeout)
 
@@ -648,8 +680,10 @@ def cli():
                         help="set log file output")
     parser.add_argument('-t', '--timeout', default=CONNECTION_TIMEOUT,
                         help="connection timeout in millisecond")
-    parser.add_argument('-pwd', '--password', default='admin',
+    parser.add_argument('-apwd', '--admin-password', default='admin',
                         help="session admin password")
+    parser.add_argument('-spwd', '--server-password', default=None,
+                        help="session server password")
 
     parser.add_argument('--attached',
                         help="server attached to a blender instance",
@@ -676,7 +710,8 @@ def cli():
     shell = ServerShell(
         port=int(args.port),
         timeout=int(args.timeout),
-        password=str(args.password),
+        server_password=str(args.server_password),
+        admin_password=str(args.admin_password),
         attached=bool(args.attached))
     try:
         shell.cmdloop()
